#!/bin/env dls-python
import PandATest
from collections import OrderedDict


class Encoder(PandATest.PandATest):

    def __init__(self, *args, **kwargs):
        super(Encoder, self).__init__(*args, **kwargs)

    def test_code(self):

        #(cmd, (expected result, +- bound of result))
        setup_quad_commands = OrderedDict([
            ('INENC{}.BYPASS=On'.format(self.blocknum),            ('OK', 0)),
            ('OUTENC{}.BYPASS=On'.format(self.blocknum),           ('OK', 0)),
            ('INENC{}.PROTOCOL=Quadrature'.format(self.blocknum),  ('OK', 0)),
            ('OUTENC{}.PROTOCOL=Quadrature'.format(self.blocknum), ('OK', 0))
        ])
        setup_ssi_commands = OrderedDict([
            ('INENC{}.PROTOCOL=SSI'.format(self.blocknum),  ('OK', 0)),
            ('OUTENC{}.PROTOCOL=SSI'.format(self.blocknum), ('OK', 0))
        ])
        quad_fields = ['A', 'B', 'Z']
        quad_channel_connect = {
            quad_fields[0]: quad_fields[0],
            quad_fields[1]: quad_fields[1],
            quad_fields[2]: quad_fields[2],
        }
        ssi_fields = ['CLK', 'DATA']

        #SET FOR QUADRATURE
        for cmd, expected in setup_quad_commands.items():
            response = self.send_cmd(cmd)
            self.test_result.append(
                self.check_response(response, expected[0], cmd, expected[1]))

        #TOGGLE QUAD FIELDS
        for outfield in quad_fields:
            cmd = "OUTENC{}.{}=BITS.ONE".format(self.blocknum, outfield)
            self.set_output(cmd)
            for infield in quad_fields:
                cmd = "INENC{}.{}?".format(self.blocknum, infield)
                self.test_result.append(
                    self.check_input(self.blocknum, "OUTENC", "INENC", cmd, infield, outfield, 1))
            cmd = "OUTENC{}.{}=BITS.ZERO".format(self.blocknum, outfield)
            self.set_output(cmd)
            for infield in quad_fields:
                cmd = "INENC{}.{}?".format(self.blocknum, infield)
                self.test_result.append(
                    self.check_input(self.blocknum, "OUTENC", "INENC", cmd, infield, outfield, 0))

        #SET FOR SSI
        for cmd, expected in setup_ssi_commands.items():
            response = self.send_cmd(cmd)
            self.test_result.append(
                self.check_response(response, expected[0], cmd, expected[1]))

        #TOGGLE SSI FIELDS
        ok_string = {
            0: "..NOT OK",
            1: "..OK"
        }
        #CLK
        cmd = "INENC{}.{}=BITS.ONE".format(self.blocknum, 'CLK')
        self.set_output(cmd)

        cmd = "OUTENC{}.{}?".format(self.blocknum, 'CLK')
        self.test_result.append(
            self.check_input(self.blocknum, "INENC", "OUTENC", cmd, 'CLK', 'CLK', 1))

        cmd = "INENC{}.{}=BITS.ZERO".format(self.blocknum, 'CLK')
        self.set_output(cmd)

        cmd = "OUTENC{}.{}?".format(self.blocknum, 'CLK')
        self.test_result.append(
            self.check_input(self.blocknum, "INENC", "OUTENC", cmd, 'CLK', 'CLK', 0))

        #DATA
        cmd = "OUTENC{}.{}=BITS.ONE".format(self.blocknum, 'DATA')
        self.set_output(cmd)

        cmd = "INENC{}.{}?".format(self.blocknum, 'DATA')
        self.test_result.append(
            self.check_input(self.blocknum, "INENC", "OUTENC", cmd, 'DATA', 'DATA', 1))

        cmd = "OUTENC{}.{}=BITS.ZERO".format(self.blocknum, 'DATA')
        self.set_output(cmd)

        cmd = "INENC{}.{}?".format(self.blocknum, 'DATA')
        self.test_result.append(
            self.check_input(self.blocknum, "INENC", "OUTENC", cmd, 'DATA', 'DATA', 0))


        return not all(self.test_result)

    def check_response(self, response, expected, cmd, bnd=0):
        try:
            response_val = int(response.split('=')[1])
            expected_val = int(expected.split('=')[1])
            check = expected_val - bnd <= response_val <= expected_val + bnd
        except IndexError:
            #if the response is simply "OK"
            check = response == expected

        if check:
            self.logger.info("{:<29} [{:<9}]  ..OK".format(cmd, response))
            return 1
        else:
            self.logger.info("{:<29} [{:<9}]  ..NOT OK".format(cmd, response))
            return 0

    def set_output(self, cmd):
        response = self.send_cmd(cmd)
        if response.strip('\n') != 'OK':
            self.logger.info("ERROR SENDING: %s", cmd)

    def check_input(self, blocknum, oblock, iblock, cmd, tst_inp, expd_inp, expd_rslt):
        response = self.send_cmd(cmd)
        ok_string = {
            0: "..NOT OK",
            1: "..OK"
        }
        if tst_inp == expd_inp:
            if response == 'OK ={}'.format(expd_rslt):
                ret = 1
            else:
                ret = 0
        else:
            #assume start point of 0 for all outputs.
            #make sure nothing toggles that isn't supposed to
            if response == 'OK ={}'.format(0):
                ret = 1
            else:
                ret = 0
        self.logger.info("{:<30}[{:<9}]  {}".format(
            "{}{}.{}=>{}, {}{}.{}=>".format(oblock, blocknum, expd_inp, expd_rslt, iblock, blocknum, tst_inp),
            response, ok_string[ret]))
        return ret
