#!/bin/env dls-python
import time, sys, os
import logging


class PandATest(object):
    def __init__(self, socket, blocknum = 0):
        self.socket = socket
        self.logger = self.setup_logging()
        self.test_result = []
        self.blocknum = blocknum

    def setup_logging(self):
        name = '.'.join([self.__class__.__name__])
        rootLogger = logging.getLogger(name)
        return rootLogger

    def run(self):
        self.get_user_input()
        self.logger.info("TEST START")
        result = self.test_code()

    def send_cmd(self, cmd):
        self.socket.sendall(cmd + '\n')
        result = self.socket.recv(4096)
        return result.strip('\n')

    def test_code(self):
        pass

    def check_input(self, in_type, outp, outp_val, tst_inp, expd_inp, expd_rslt, unplugged_result=0):
        cmd = in_type + str(tst_inp) + '.VAL?'
        response = self.send_cmd(cmd)
        ok_string = {
            0: ".. NOT OK",
            1: ".. OK"
        }
        if tst_inp == expd_inp:
            if response == 'OK ={}'.format(expd_rslt):
                ret = 1
            else:
                ret = 0
        else:
            #assume start point of 0 for all outputs.
            #make sure nothing toggles that isn't supposed to
            if response == 'OK ={}'.format(unplugged_result):
                ret = 1
            else:
                ret = 0

        self.logger.info("{:<21}[{:<9}]  {}".format(
            "OUT{}=>{}, INP{}=>".format(outp, outp_val, tst_inp), response, ok_string[ret]))
        return ret

    def set_output(self, out_type, output, val, expected):
        cmd = out_type + str(output) + val
        response = self.send_cmd(cmd)
        if response.strip('\n') != 'OK':
            self.logger.info("ERROR SENDING: %s", cmd)

    def get_summary(self):
        error_count = self.test_result.count(0)
        summary = ["{:<10}{:<10} ERRORS: {}".format(
                                                self.__class__.__name__,
                                                self.blocknum,
                                                error_count)]
        return summary

    def get_user_input(self):
        pass